// Module: IntervalRebinner.h
// Author: Hassan Siddiqui
//
// Description: 
// Rebins the ranges defined by an input BinnedAxis object
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: IntervalRebinner.cc,v 1.1 2000/03/28 12:42:50 hsiddiqu Exp $

/* General classes */

/* SAS-specific classes */
#include "errstr.h"
#include "Tracer.h"

/* Local classes */
#include "IntervalRebinner.h"
#include "NonUnifBinnedAxis.h"

/***********************************************************
 *
 * class IntervalRebinner
 *
 ***********************************************************/


/* Constructor */
IntervalRebinner::IntervalRebinner()
{
  PROC;
  ENDPROC;
}

/* Destructor */
IntervalRebinner::~IntervalRebinner(){};

/* Return a object containing the resampled intervals, 
 * based on an input BinnedAxis object. */ 
IntervalVector<double> 
IntervalRebinner::rebin( 
			const BinnedAxis * in, 
			unsigned binfactor,
			unsigned offset)
{
  PROC;

  unsigned npixels = in->bins();

  errstr << "IntervalRebinner::rebin: got "
	 << " binfactor=" << binfactor
	 << " offset=" << offset
	 << " npixels=" << npixels
	 << Sas::message(Msg::App, Msg::Verbose);
  
  IntervalVector<double> intvector;

  unsigned loweridx = offset;
  unsigned upperidx;
  double low;
  double high;

  while (loweridx < npixels)
    {
      low = in->bin(loweridx).lo();

      upperidx = loweridx + binfactor-1;
      if (upperidx >= npixels)
	{
	  upperidx = npixels - 1;
	}
      high = in->bin(upperidx).hi();

      intvector.push_back(Bin(low,high));
      loweridx += binfactor;
    }

  /* Check size of output vector */
  unsigned expectedsize = (npixels-offset)/binfactor;
  expectedsize = 
    ((expectedsize*binfactor) == npixels-offset) 
    ? expectedsize : expectedsize+1;

  if (intvector.size() != expectedsize)
    {
      errstr << "Number of bins in output ["<< intvector.size()
	     << "] does not match the number expected ["<< expectedsize
	     << "]" 
	     << Sas::error("IntervalRebinnerInvalidOutput"); 
    }

  RETURN intvector;
}




